<?php
/**
 * @package   Switips
 * @author    Outcode https://outcode.ru/
 * @license   http://www.gnu.org/licenses/gpl-2.0.html GNU/GPLv2 only
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

//jbdump($params);

class plgHikashopSwitips extends JPlugin{

  public $states = [    // Статусы Hikashop
    'created' => 'new',        // НОВЫЙ
    'confirmed' => 'break',    // ПОДТВЕРЖДЕНО
    'cancelled' => 'canceled', // ОТМЕНЕН 
    'refunded' => 'canceled',  // ВОЗВРАЩЕН
    'shipped' => 'break',      // ОТПРАВЛЕН 
    'pending' => 'break',      // В ОЖИДАНИИ 
  ];
  
  public function __construct(&$subject, $config){
    parent::__construct($subject, $config);
  }
  
  public function onAfterOrderUpdate(&$order, &$send_email){
    $model = hikashop_get('class.order');
    $order_full = $model->loadFullOrder($order->order_id); 
    
    $app = &JFactory::getApplication();
    
    if($order_full->order_status == 'created'){    
      $uid = $app->input->cookie->get('switips_uid', '');
    }else{
      $uid = $this->getUid($order_full->order_id);
    }    

    $commission = $this->getCommission($order_full->products);    
    $result = $this->sendToSwitips($uid, $order_full, $commission);

   }  
  
  public function sendToSwitips($uid, $order, $commission = 0){
    $switips = $this->getParams();

    if($uid == ''){ return; }
    if(intval($switips->sw_status) != 1){ return; }     
    if($this->states[$order->order_status] == 'break'){ return; }
    
    $currency = unserialize($order->order_currency_info);

    $args = [      
      'secret_key' => $switips->key_api,
      'merchant_id' => $switips->merchant_id,
      'campaign_id' => 0,
      'category_id' => 0,
      'user_id' => $uid,    
      'transaction_id' => $order->order_id,
      'transaction_amount' => $order->order_full_price,
      'transaction_amount_currency' => $order->order_full_price,
      'currency' => $currency->currency_code,
      'tt_date' => $order->order_created,
      'stat' => $this->states[$order->order_status], 
      'commission_amount' => $commission,
    ];

    JLoader::register('Switips', JPATH_ROOT . '/libraries/switipsapi/switips.php');    
    $SwitipsCls = new Switips($switips->url_api);    
    $send = $SwitipsCls->sendRequestPost($args);
    $this->saveUid($uid, $order->order_id);

    unset($args['secret_key']);
    unset($args['merchant_id']);
    
    $this->logs($args, 'args');
    $this->logs($send, 'send');

    return $send;
  }  
  
  function getCommission($products){
    $db = JFactory::getDBO();

    $return = 0;

    foreach($products as $product){
      $qty = $product->order_product_quantity;
      $unit_price = $product->order_product_price; // Цена одной единицы товара
      
      $commission = $this->categoryCommissions($product->product_id, $unit_price);        
      
      $query = 'SELECT `commission` FROM  #__hikashop_product WHERE `product_id`='.intval($product->product_id);
      $db->setQuery($query);
      $result = $db->loadAssocList();

      $tmp = (isset($result[0]['commission'])) ? $result[0]['commission'] : 0;
      $product_commission = $this->checCommission($tmp);      

      if($product_commission > 0){
        
        if(strpos($product_commission, '%') !== false){
          $product_commission = str_replace('%', '', $product_commission);
          $product_commission = ($unit_price/100)*$product_commission;          
        }
        
        $commission = $product_commission;        
      }

      $return += $qty * $commission;
    }

    return $return;
  }
  
  
  // Отдает максимальное значение комиссии из категорий
  public function categoryCommissions($product_id, $unit_price){
    $db = JFactory::getDBO();

    $query = "SELECT pr_cat.category_id, cat.commission FROM `#__hikashop_product_category` AS pr_cat
              LEFT JOIN `#__hikashop_category` AS cat ON pr_cat.category_id = cat.category_id
              WHERE pr_cat.product_id=".(int)$product_id;
    $db->setQuery($query);
    $data = $db->loadAssocList();
    
    $catalog_commission = 0;
    $prev_commission = 0;
    $next_commission = 0;
    
    foreach($data as $d){
      $next_commission = $this->checCommission($d['commission']);
      
      if($next_commission == 0){
        continue;
      }

      if(strpos($next_commission, '%') !== false){
        $next_commission = str_replace('%', '', $next_commission);
        $next_commission = ($unit_price/100)*$next_commission; 
      }      
      
      // max значение
      $catalog_commission = ($next_commission > $prev_commission) ? $next_commission : $prev_commission;
      $prev_commission = $catalog_commission;
    }
    
    return $catalog_commission;
  }
  
  public function checCommission($value = 0){
    $pattern = '/[^0-9 %]+/msiu';
    if(preg_match($pattern, $value)){      
      return 0; // Найдены запрещенные символы
    }else{
      return str_replace(' ', '', $value);
    }  
  }  
  
  public function saveUid($uid, $category_id = 0){
    $db = &JFactory::getDBO();
    $data = new stdClass();
    $data->order_id = $category_id;
    $data->uid = $uid;

    $result = $db->insertObject('#__switips_uid', $data);
    return $result;
  }
  
  public function getUid($order_id){
    $db = &JFactory::getDBO();
    
    if(!$db->connected()) {
      return null;
    }else{  

      $query = 'SELECT `uid` FROM  #__switips_uid WHERE `order_id`='.intval($order_id);
      $db->setQuery($query);
      $result = $db->loadAssocList();  
      
      if(isset($result[0]['uid'])){
        return $result[0]['uid'];
      }else{
        return '';
      }
      
    } 
  } 
  
  public function getParams(){
    $db = &JFactory::getDBO();
  
    $query = "SELECT `params` FROM  #__extensions WHERE `element`='com_switips'";
    $db->setQuery($query);
    $result = $db->loadAssocList(); 
    
    if(isset($result[0]['params'])){
      return json_decode($result[0]['params']);
    }       
  }

  public function logs($arg, $suf){
    $file = __DIR__.'/'.date('d.m.y', time()).'--'.$suf.'.txt';
    
    if(file_exists($file)){
      $current = file_get_contents($file);
    }else{
      $current = '';
    }

    $current .= '========='.date('d.m.y-h:m:s', time()).'==========='.PHP_EOL;
    $current .= var_export($arg, true).PHP_EOL;
    file_put_contents($file, $current);     
  }

}


